var fmt = require('./format')
var fs = require('fs')
var toml = require('toml-js')
var needle = require('needle')
var Promise = require('bluebird')

// validates the existence of a crate in the official crates.io index, which
// luckily, is hosted over at github. the crates.io website returns REALLY weird
// status codes
function validateCrate (cName) {
  var f2 = cName.substr(0, 2)
  var m2 = cName.substr(2, 2)
  var query = 'https://raw.githubusercontent.com/rust-lang/crates.io-index/master/' + f2 + '/' + m2 + '/' + cName

  return new Promise(function (resolve, reject) {
    needle.get(query, function (err, res) {
      if (res.statusCode === 404) {
        reject(cName + ' is not in the crates.io index! (' + err + ')')
      }
      resolve()
    })
  })
}

module.exports = function (args) {
  Promise.promisifyAll(fs)
  fmt.normal('checking for problems...')
  // convenience variable for less typing
  var cName = args._[1]
  // 1. check if there is a crate name supplied
  if (!cName) {
    fmt.error('no crate name supplied!')
  }
  // 2. check if the crate exists on crates.io
  // this is very complicated at the moment, which is why it's in a separate
  // function
  validateCrate(cName)
    .then(function () {
      fmt.normal('installing ' + cName + '...')
      // 3. load the contents of the local Cargo.toml
      return fs.readFileAsync(process.cwd() + '/Cargo.toml', 'utf-8')
    }, function (e) {
      fmt.error(e)
    }).then(function (data) {
      // 4. parse the toml to json and add the dependency
      var cToml = toml.parse(data)
      if (!cToml.dependencies) {
        cToml.dependencies = {}
      }
      cToml.dependencies[cName] = '*'
      var result = toml.dump(cToml)
      // 5. write the toml back to the Cargo.toml!
      return fs.writeFileAsync(process.cwd() + '/Cargo.toml', result, 'utf-8')
    }, function (e) {
      fmt.error(e)
    }).then(function () {
      fmt.success(cName + ' has been added to your Cargo.toml!')
    }, function (e) {
      fmt.error(e)
    }).catch(function (e) {
      fmt.error(e)
    })
}
